/*
Author: Dominic Nicholson
Description:
This program is designed to display an interactive pie chart which allows the 
user to see the capacity and current enrollment of schools, departments and classes.
It does this by creating a 2-3 seperate pie charts. The innermost pie will is 
a simple back button. The middle pie shows either individual classes or subjects
depending on 'Pie_ZoomLevel'. The outer pie is used to group the classes or subjects of
the middle pie into subjects and classes respectively. At the highest zoom level the outer
pie ring is not needed.
*/

var Pie_hostname = "http://dominicnicholson.no-ip.biz/RegVis/";
queue()
	.defer(d3.csv, Pie_hostname + "CourseTotals.csv")
	.defer(d3.csv, Pie_hostname + "Students.csv")
	.defer(d3.csv, Pie_hostname + "PieReg.csv")
	.await(InitPie);

var Pie_allData = [];				//Stores all class data
var Pie_innerPie,Pie_midPie,Pie_outerPie, Pie_backText;	//Stores svg elements for later clearing
var Pie_LastFilteredSchool,Pie_LastFilteredColor;		//Stores data from last pie generation
var Pie_container,Pie_SubTitle;		//Sotes the container and subtile objects
var Pie_RADIUS = 300;				//Set the radius of the main pie
var Pie_INNERRADIUS = 80;			//Sets the radius of the back button
var Pie_PADDING = 100;				//Sets the size of the padding area around the pie
var Pie_EDGEWIDTH = 20;				//Sets the width of the outer ring

var Pie_ZoomLevel = 0;				//Current filter level
var Pie_RandomSeed = 1;	

var Pie_CurrentTooltipData;
var Pie_TooltipInitialPos = {"x":0,"y":0};
var Pie_TooltipStartDrag = {"x":0,"y":0};


var Pie_Students = [];
var Pie_SIDs = [];
var Pie_Enrollment = [];



function InitPie(error, Data,rawStudents, rawReg) {
	if (error) throw error;


	rawStudents.forEach(
		function (d) {
			Pie_SIDs.push(d["student_id"]);
			Pie_Students[d["student_id"]] = {
				"name":(d["student_fName"] + " " + d["student_lName"]),
				"gender":d["gender"],
				"ethnicity":d["ethnicity"],
			};
		}
	);

	rawReg.forEach(function(d) {
		if (typeof Pie_Enrollment[d["class_number"]] === 'undefined')
			Pie_Enrollment[d["class_number"]] = [];
		Pie_Enrollment[d["class_number"]].push({"id":d["student_id"],"grade":d["grade"]});

		if (typeof Pie_Enrollment[d["subject"]] === 'undefined') 
			Pie_Enrollment[d["subject"]] = {"classnumbers":new Set(), "classes":[]};

		if (typeof Pie_Enrollment[d["subject"]].classes[d["class_number"]] === 'undefined') {
			Pie_Enrollment[d["subject"]].classes[d["class_number"]] = {"name":d["catalog"], "section":d["section"],"sum":0};
		}

		Pie_Enrollment[d["subject"]].classnumbers.add(d["class_number"]);
		++Pie_Enrollment[d["subject"]].classes[d["class_number"]].sum;


	})

	//Sort in a way that makes for easy filtering and diplay
	Data.sort(
		function (a,b) {
			if (a["school_name"] < b["school_name"]) return -1; 
			else if (a["school_name"] == b["school_name"]) {
				if (a["subject"] < b["subject"]) return -1;
				else if (a["subject"] == b["subject"]) {
					if (parseInt(a["catalog"]) < parseInt(b["catalog"])) return -1;
					else if (parseInt(a["catalog"]) == parseInt(b["catalog"])) return 0;
					return 1;
				}
				return 1;
			}
			return 1;
		}
	);

	//Remove duplicate data
	Pie_allData = Data.filter(function (d,i) {
		if (d.enrol_capacity > 5 && d.total_enrolled > 0) {
			d.total_enrolled = d.enrol_capacity - d.total_enrolled;

			if (typeof Pie_Enrollment[d["subject"]] != 'undefined' && typeof Pie_Enrollment[d["subject"]].classes[d["class_number"]] != 'undefined') {
				Pie_Enrollment[d["subject"]].classes[d["class_number"]].capacity = d.enrol_capacity;
			}

		    return d;
	    }
		else return;
		
	});

	//Create the subtile and container objects
	Pie_SubTitle = d3.select("body").append("svg:svg")
		.attr("width",Pie_RADIUS*2)
		.attr("height",50)
		.append("svg:text")
		.attr("transform", "translate(10,40)")
		.attr("text-anchor", "left")
		.attr("class", "Title")
	;
	d3.select("body").append("br");
	Pie_container = d3.select("body").append("svg:svg");
	Pie_container.attr("width", Pie_RADIUS*2 + Pie_PADDING*2 + Pie_EDGEWIDTH*2)
		.attr("height", Pie_RADIUS*2 + Pie_PADDING*2 + Pie_EDGEWIDTH*2)

	RenderPie(Pie_allData,0);

}

function ReFilterBy(data,sortKey, target,color) {
	//Filter the data and re-create the pie
	var filtered = data.filter(function (d) {
		if (d[sortKey] == target) return d;
		else return;
	});

	//Alter the subtile to reflect changes
	if (Pie_ZoomLevel == 2)
		Pie_SubTitle.text("" + target + " Department View");
	else Pie_SubTitle.text("School of " + target + " View");

	RenderPie(filtered,color);
}
function RenderPie(data,StartColor) {
	//Set up all parts of the pie diagram for rendering

	//Clear all data
	if (Pie_innerPie) {
		Pie_innerPie.remove();
		Pie_midPie.remove();
		Pie_outerPie.remove();
		Pie_backText.remove();
	}
	var lastSubject = "";
	var lastSchool = "";

	var subjectData = [];	//Aggregated data by subject
	var schoolData = [];	//aggregated data by school

	//Create subject and school data aggregates
	for (i = 0; i < data.length; ++i) {

		if (lastSubject != data[i]["subject"]) {
			subjectData.push({
				"subject":data[i]["subject"],
				"school_name":data[i]["school_name"],
				"catalog":data[i]["subject"],
				"class_number":data[i]["class_number"],
				"total_enrolled":Number(data[i]["total_enrolled"]),
				"enrol_capacity":Number(data[i]["enrol_capacity"]),
				"Radius":-1
			});
			lastSubject = data[i]["subject"];
		}
		else {
			subjectData[subjectData.length-1]["total_enrolled"] += Number(data[i]["total_enrolled"]);
			subjectData[subjectData.length-1]["enrol_capacity"] += Number(data[i]["enrol_capacity"]);
		}

		if (lastSchool != data[i]["school_name"]) {
			schoolData.push({
				"subject":data[i]["school_name"],
				"school_name":data[i]["school_name"],
				"catalog":data[i]["catalog"],
				"total_enrolled":Number(data[i]["total_enrolled"]),
				"enrol_capacity":Number(data[i]["enrol_capacity"]),
				"Radius":-1
			});
			lastSchool = data[i]["school_name"];
		}
		else {
			schoolData[schoolData.length-1]["total_enrolled"] += Number(data[i]["total_enrolled"]);
			schoolData[schoolData.length-1]["enrol_capacity"] += Number(data[i]["enrol_capacity"]);
		}
	}

	//BackButton
	Pie_innerPie = createCircle([{
			"subject":"Back",
			"school_name":"",
			"catalog":0,
			"total_enrolled":1 - (Pie_INNERRADIUS/Pie_RADIUS),
			"enrol_capacity":1,
			"Radius":-1
		}], Pie_container, Pie_RADIUS, 0, Pie_PADDING, "enrol_capacity", "total_enrolled", "subject" ,false, false,StartColor)
		.attr("Class", "Pie")
	;

	Pie_backText = Pie_container.append("svg:text")
		.attr("transform", 
			function(d) {
				return "translate(" + (Pie_RADIUS + Pie_PADDING + Pie_EDGEWIDTH) + "," + (Pie_RADIUS + Pie_PADDING + Pie_EDGEWIDTH+6) + ")";
			}
		)
		.attr("text-anchor", "middle")
		.attr("class", "ButtonText")
	;


	//Overview
	if (Pie_ZoomLevel == 0) {
		Pie_midPie = createCircle(subjectData, Pie_container, Pie_RADIUS, Pie_INNERRADIUS, Pie_PADDING, "enrol_capacity", "total_enrolled", "subject", false, true,StartColor);
		Pie_outerPie = createCircle(schoolData, Pie_container, Pie_RADIUS+Pie_EDGEWIDTH, Pie_RADIUS, Pie_PADDING, "enrol_capacity", "Radius", "school_name", true, true,StartColor);
		Pie_backText.text("SSU");
		Pie_SubTitle.text("Overview");
	}
	//Department view
	else if (Pie_ZoomLevel == 1) {
		Pie_midPie = createCircle(data, Pie_container, Pie_RADIUS, Pie_INNERRADIUS, Pie_PADDING, "enrol_capacity", "total_enrolled", "subject", false, true,StartColor);
		Pie_outerPie = createCircle(subjectData, Pie_container, Pie_RADIUS+Pie_EDGEWIDTH, Pie_RADIUS, Pie_PADDING, "enrol_capacity", "Radius", "subject", true, true,StartColor);
		Pie_backText.text("Overview");
	}
	//Subject view
	else {
		Pie_midPie = createCircle(data, Pie_container, Pie_RADIUS, Pie_INNERRADIUS, Pie_PADDING, "enrol_capacity", "total_enrolled", "catalog", true, false,StartColor);
		Pie_backText.text(Pie_LastFilteredSchool);
	}

}
function createCircle(data, Pie_container, radius, innerRad, padding, sliceKey, radiusKey, labelKey, DrawText, AppliesFilters, StartColor) {
	//Create the d3js for a single pie chart
	Pie_RandomSeed = 1;	//Reset the random seed to keep colors constant

	var colorScales =[
		//light blue/blue
		d3.scale.linear()
			.interpolate(d3.interpolateHcl) 
			.range(["#37A9BA","#3C86AD"]),
		//orange/red
		d3.scale.linear()
			.interpolate(d3.interpolateHcl) 
			.range(["#CC7766", "#FF7744"]),
		//green/yellow
		d3.scale.linear()
			.interpolate(d3.interpolateHcl) 
			.range(["#77CC00","#EEEE00"])
	];

	Pie_container.data([data]);

	var vis = Pie_container.append("svg:g");	//container for this pie

	//If it's the outer circle it needs a different translation
	if (AppliesFilters && DrawText) 
		vis.attr("transform", "translate(" + (radius + padding) + "," + (radius + padding) + ")");
	
	else vis.attr("transform", "translate(" + (radius + padding+Pie_EDGEWIDTH) + "," + (radius + padding+Pie_EDGEWIDTH) + ")");

	//layout the pie based on sliceKey, and don't sort it based on slice size 
	var pie = d3.layout.pie()
		.value(
			function(d) {
				return d[sliceKey];
			}
		)
		.sort(
			null
		)
	;

	//Create a pie slice for each data point
	var arcs = vis.selectAll("g.slice")
		.data(pie)
		.enter()
			.append("svg:g")
				.attr("class", "slice")
	;

	var LastSubject;
	var LastColor = StartColor % 3;

	//Create the drawn data for each pie slice
	arcs.append("svg:path")
		.attr("fill", 
			function(d, i) {
				if (Pie_ZoomLevel > 0) {
					if (data[i]["subject"] == LastSubject) return colorScales[LastColor](random());

					LastSubject = data[i]["subject"];
					++LastColor;
					if (LastColor == colorScales.length) LastColor = 0;

					return colorScales[LastColor](random());
				}
				else {
					if (data[i]["school_name"] == LastSubject) return colorScales[LastColor](random());

					LastSubject = data[i]["school_name"];
					++LastColor;
					if (LastColor == colorScales.length) LastColor = 0;

					return colorScales[LastColor](random());
				}
			}
		)
		.attr("d", 
			//Set the radius of each slice, to indicate the 'fullness' of the class it represents
			d3.svg.arc().outerRadius(
				function(d, i) {
					if (data[i][radiusKey] == -1) return radius;
					return (
						(
							(data[i][sliceKey] - data[i][radiusKey])
							/ 
							data[i][sliceKey]
						) * ((1 - (innerRad/radius)))
						* (radius) + innerRad
						)
					;
				}
			).innerRadius(innerRad)
		)
		.on("mouseenter", function (d,i) {return MakeTooltip(data[i])})	//Add a tooltip on mouseover
		.on("mouseleave", function () {ourTooltip();})					//remove tooltip
		.on("click", function (d,i) {
			ourTooltip();	//remove tooltip when clicked

			//if this is a back button, zoom out
			if (data[i]["subject"] == "Back") {
				--Pie_ZoomLevel;
				if (Pie_ZoomLevel < 0) Pie_ZoomLevel = 0;
				if (Pie_ZoomLevel == 0)
					RenderPie(Pie_allData,0);

				else if (Pie_ZoomLevel == 1) ReFilterBy(Pie_allData,"school_name",Pie_LastFilteredSchool,Pie_LastFilteredColor);
			}
			//Zoom in and apply filters
			else if (AppliesFilters) {
				if (DrawText) {
					++Pie_ZoomLevel;
					if (Pie_ZoomLevel > 2)
						Pie_ZoomLevel = 2;
				}
				if (Pie_ZoomLevel == 1) {
					if (DrawText) {
						Pie_LastFilteredSchool = data[i]["school_name"];
						Pie_LastFilteredColor = i;
						ReFilterBy(Pie_allData,"school_name",data[i]["school_name"],i);
					}
					else {
						var SD = [];
						SD.attributes = [];
						SD.attributes.push({"name":"Student ID", "key": "id"});
						SD.attributes.push({"name":"Name", "key": "name"});
						SD.attributes.push({"name":"Grade", "key": "grade"});

						if (typeof Pie_Enrollment[data[i]["class_number"]] === 'undefined') {
							console.log("The class number " + data[i]["class_number"] + " does not exist in the registration file");
							return;
						}

						Pie_Enrollment[data[i]["class_number"]].forEach(function(d) {
							var S = Pie_Students[d.id];
							SD.push({
								"id":d.id,
								"name":S["name"],
								"grade":d.grade, 
								"gender":S["gender"],
								"ethnicity":S["ethnicity"]
							});
						});

						SD.name = data[i]["subject"] + " " +  data[i]["catalog"];
						SD.dataname = "students";

						Tooltip(SD);
					}
				}
				else {
					if (DrawText) {
						ReFilterBy(Pie_allData,"subject",data[i]["subject"],i);
					}
					else {
						var SD = [];
						SD.attributes = [];
						SD.attributes.push({"name":"Course ID", "key": "id"});
						SD.attributes.push({"name":"Catalog", "key": "catalog"});
						SD.attributes.push({"name":"Section", "key": "section"});
						SD.attributes.push({"name":"Enrollment", "key": "enrollment"});
						SD.attributes.push({"name":"Capacity", "key": "capacity"});

						if (typeof Pie_Enrollment[data[i]["subject"]] === 'undefined') {
							console.log("The subject " + data[i]["subject"] + " does not exist in the registration file");
							return;
						}

						Pie_Enrollment[data[i]["subject"]].classnumbers.forEach(function(d) {
							var S = Pie_Enrollment[data[i]["subject"]].classes[d];
							SD.push({
								"id":d,
								"catalog":S.name,
								"section":S.section,
								"enrollment":S.sum,
								"capacity":S.capacity,
								"grade":null, 
								"gender":null,
								"ethnicity":null
							});
						});

						SD.name = data[i]["subject"];
						SD.dataname = "courses";

						Tooltip(SD);

					}
				}
			}
			else {
				var SD = [];
				SD.attributes = [];
				SD.attributes.push({"name":"Student ID", "key": "id"});
				SD.attributes.push({"name":"Name", "key": "name"});
				SD.attributes.push({"name":"Grade", "key": "grade"});

				if (typeof Pie_Enrollment[data[i]["class_number"]] === 'undefined') {
					console.log("The class number " + data[i]["class_number"] + " does not exist in the registration file");
					return;
				}

				Pie_Enrollment[data[i]["class_number"]].forEach(function(d) {
					var S = Pie_Students[d.id];
					SD.push({
						"id":d.id,
						"name":S["name"],
						"grade":d.grade, 
						"gender":S["gender"],
						"ethnicity":S["ethnicity"]
					});
				});

				SD.name = data[i]["subject"] + " " +  data[i]["catalog"];
				SD.dataname = "students";

				Tooltip(SD);
			}
		})
	;
	//Add text labels to each pie slice if needed
	if (DrawText) {
		arcs.append("svg:text")
			.attr("transform", 
				function(d) {
					if (Pie_ZoomLevel == 0) {
						var rotation = 
							(
								(
									(d.startAngle + d.endAngle)
									/
									(4*Math.PI)
								) 
							) * 360
						;
						return "rotate(" 
							+ 
							rotation
							+ ") translate(0," + (-radius - 4) + ")"
						;
						
					}
					var rotation = 
						(
							(
								(d.startAngle + d.endAngle)
								/
								(4*Math.PI)
							) -.25
						) * 360
					;
					if (rotation > 90 && rotation < 270) {
						return "rotate(" 
							+ 
							(360 - (180 - rotation))
							+ ") translate(" + (-radius - 4) + ",0)"
						;
					}
					else {
						return "rotate(" 
							+ 
							rotation
							+ ") translate(" + (radius + 4) + ",0)"
						;
					}
				}
			)
			.attr("text-anchor", function(d) {
				if (Pie_ZoomLevel == 0) {
					return "middle";
					
				}
				var rotation = 
					(
						(
							(d.startAngle + d.endAngle)
							/
							(4*Math.PI)
						) -.25
					) * 360
				;
				if (rotation > 90 && rotation < 270) return "end";
				
				else return "start";
				
			})
			.text(
				function(d, i) { 
					return data[i][labelKey];
				}
			)
			.on("mouseenter", function (d,i) {return MakeTooltip(data[i])})
			.on("mouseleave", function () {ourTooltip();})
		;
	}
	return vis;
}
function MakeTooltip (d) {
	//Helper function for making tooltips based on zoom level
	if (d["subject"] == "Back") return;
	var temp1 = [];
	var temp2 = [];
	if (Pie_ZoomLevel == 0) {
		temp1 = [d["subject"], d["total_enrolled"] ,d["enrol_capacity"]];
		temp2 = ["", "Available Seats : ", "Capacity : "];
	}
	else if (Pie_ZoomLevel == 1) {
		temp1 = [d["catalog"], d["total_enrolled"] ,d["enrol_capacity"]];
		temp2 = ["", "Available Seats : ", "Capacity : "];
	}
	else {
		temp1 = [d["catalog"], d["total_enrolled"] ,d["enrol_capacity"]];
		temp2 = [d["subject"] + " " ,"Available Seats : ", "Capacity : "];
	}
	ourTooltip(temp1, temp2);
}

function random() {
	//helper function that generates a random number based on Pie_RandomSeed
    var x = Math.sin(Pie_RandomSeed++) * 10000;
    return x - Math.floor(x);
}

function Tooltip(SD,SortBy) {
	//This function creates an info window that allows the user to select students to filter by
	//and shows relevant info in pie charts

	//Save old tooltip data in case we need it
	var div = document.getElementById('tooltip');
	var x,y;
	if (typeof div != 'undefined' && div != null) {
		x = div.style.left;
		y = div.style.top;
	}

	//Clear the old tooltip to make foom for this new one
	d3.select('.MyTooltip').remove();

	if (typeof SD === 'undefined' || SD == null) {
		SD = Pie_CurrentTooltipData;
		SortBy = "name";
	}
	else {
		Pie_CurrentTooltipData = SD;
	}

	if (typeof SD === 'undefined' || SD == null) return;

	var base = d3.select('body').append('div').attr("class", "MyTooltip").attr("id","tooltip");

	base
		.style('display', 'block')
		.style("position","absolute")
		.data([SD])
	;

    window.addEventListener('mouseup', mouseUp, false);	//Add the stop-drag handler

    //If this is the first time generating this window, position it anew
    if (typeof SortBy === 'undefined') {
    	base
    		.style('top', (d3.event.pageY) + 'px')
			.style('left', (d3.event.pageX - 350) + 'px')
			.style("opacity", 0)
			.transition().delay(250).duration(250).style('opacity', .8)
		;
    	mouseUp();			//The lazy way to position the window		
    	SortBy = "name";
	}		
	//We don't need to run the fade-in if this is a sorting event
	else {
		base
			.style('top', y)
			.style('left', x)
			.style("opacity", .8)
		;
	}	

	SD.sort(function (a,b) {
		if (a[SortBy] < b[SortBy]) return -1;
		else if (a[SortBy] == b[SortBy]) 0;
		return 1;
	});

	base.on("mouseover",function() {
			this.style["opacity"] = ".8";
		})
		.on("mouseout",function() {
			this.style["opacity"] = ".3";
		})
	;


	var table = base.append('table').style("background-color","#000000");
	var row1 = table.append("tr").attr("id","tooltip-toolbar").attr("height",30).style("background-color","#EEF0FF");	//The title bar for the tooltip window

	//Add a listener to allow the toolbar to be dragged
	document.getElementById('tooltip-toolbar').addEventListener('mousedown', mouseDown, false);	

	//Window title
	var titlebar = row1.append("td").append("div");
	titlebar.append("text")
		.attr("class", "Title")
		.style("padding-left","10px")
		.text(SD.name)
	;
	titlebar.append("text")
		.attr("class", "Title")
		.style("padding-right","10px")
		.style("float","right")
		.text(SD.length + " " + SD.dataname)
	;

	//Close window button
	row1.append("td").append("button")
		.style("float", "right")
		.style("margin-right", "10px")
		.text("Close").on("click", function () {d3.select('.MyTooltip').remove();Pie_CurrentTooltipData = null;})
	;


	var row1,selection;

	row1 = table.append("tr").style("background-color","#FFFFFF");


	selection = row1.append("td").append("div")
		.style("width",500)
		.style("height",700)
		.style("overflow-y","scroll")
		.style("padding","10px")
	;
	var innertable = selection.append('table').style("background-color","#AAAAAA").style("width","100%");


	var TopRow = innertable.append("tr").style("background-color","#FFFFFF");
	SD.attributes.forEach(function(d) {
		TopRow.append('td').html(d.name).attr("class","SubTitle").data([d.key]);
	});
	var HoverColor = "#DDDDEE";
	TopRow.selectAll('td')
		.each(function (d) {
			var sortby = d;
			this.onclick = function () {
				
				var div = document.getElementById('tooltip');
				var x = div.style.left;
				var y = div.style.top;

				Tooltip(SD,sortby);

				div = document.getElementById('tooltip');

				div.style.left = x;
				div.style.top = y;

			};
			this.onmouseover = function () {
				this.style["background-color"] = HoverColor;
			}
			this.onmouseout = function () {
				this.style["background-color"] = "#FFFFFF";
			}
		})
	;

	var AltColor = "#F0F0FF";
	var SelectColor = "#99AAFF";
	
	//Create a list of students in this class/link and make them selectable
	SD.forEach(function(d,i) {
		var NewRow = innertable.append("tr");

		NewRow.data([d.id])
			.style("background-color",function(d) {
				
				if (i % 2) return AltColor;

				return "#FFFFFF";
			})
		;

		SD.attributes.forEach(function(attr) {
			NewRow.append('td').html(d[attr.key]);
		});
	});


	selection = row1.append("td").attr("height","100%").style("vertical-align", "top").attr("rowspan",3);

	var Charts = selection.append("svg:svg").attr("width","200px").attr("height","100%");

	var radius = 50;
	var padding = 50;

	//Titles for the charts
	Charts.append("svg:text")
		.attr("transform", "translate(" + (radius + padding) + "," + padding/2 + ")")
		.attr("text-anchor","middle")
		.text("Grades")
	;
	Charts.append("svg:text")
		.attr("transform", "translate(" + (radius + padding) + "," + ((radius + padding)*2 + padding) + ")")
		.attr("text-anchor","middle")
		.text("Gender")
	;
	Charts.append("svg:text")
		.attr("transform", "translate(" + (radius + padding) + "," + ((radius + padding)*4 + 3*padding/2) + ")")
		.attr("text-anchor","middle")
		.text("Ethnicity")
	;

	//Format the data for charting and create their respective containers
	var GradeCont = Charts.append("svg:g").attr("transform", "translate(" + (radius + padding) + "," + (radius + padding + padding/2) + ")");
	var Grades = [
		"NULL",
		"*B-",
		"*C+",
		"A",
		"A-",
		"B+",
		"B",
		"B-",
		"C+",
		"C",
		"C-",
		"CR",
		"D+",
		"D",
		"D-",
		"F",
		"I",
		"IC",
		"NC",
		"RP",
		"RPT",
		"W",
		"WU"
	];
	var GradeData = [
		{"id":"NULL","value":0},
		{"id":"*B-","value":0},
		{"id":"*C+","value":0},
		{"id":"A","value":0},
		{"id":"A-","value":0},
		{"id":"B+","value":0},
		{"id":"B","value":0},
		{"id":"B-","value":0},
		{"id":"C+","value":0},
		{"id":"C","value":0},
		{"id":"C-","value":0},
		{"id":"CR","value":0},
		{"id":"D+","value":0},
		{"id":"D","value":0},
		{"id":"D-","value":0},
		{"id":"F","value":0},
		{"id":"I","value":0},
		{"id":"IC","value":0},
		{"id":"NC","value":0},
		{"id":"RP","value":0},
		{"id":"RPT","value":0},
		{"id":"W","value":0},
		{"id":"WU","value":0}
	];
	var GenderCont = Charts.append("svg:g").attr("transform", "translate(" + (radius + padding) + "," + ((radius + padding)*3 + padding) + ")");
	var GenderData = [{"id":"Male","value":0},{"id":"Female","value":0}];
	var EthnicityCont = Charts.append("svg:g").attr("transform", "translate(" + (radius + padding) + "," + ((radius + padding)*5 + 3*padding/2) + ")");
	var Ethnicities = [
		"WHITE",
		"NULL",
		"DECLINE",
		"MEXAMER",
		"ASIAN",
		"EUROPEAN",
		"HISPA",
		"TWOMORE",
		"CENTAMER",
		"NORESPON",
		"FILIPINO",
		"BLACK",
		"CHINESE",
		"MEXICAN",
		"AMIND",
		"OTHRNWT",
		"UNKNOWN",
		"HISPANIC",
		"BLACKPRF",
		"OTHRLTHS",
		"SOAMER",
		"OTHRWHT",
		"CUBAN",
		"LAOTIAN",
		"OTHRSEAS",
		"PUERTOR",
		"OTHRPACI",
		"JAPANESE",
		"SAMOAN",
		"PACIF",
		"KOREAN",
		"CAMBOD",
		"ASINDIAN",
		"OTHRASN",
		"HAWAIIAN",
		"VIETNAME",
		"AFRAM",
		"GUAMANIA",
		"THAI",
		"DECHISPA",
		"OTHER",
		"OTHRAMIN",
		"HMONG"
	];
	var EthnicityData = [
		{"id":"WHITE","value":0},
		{"id":"NULL","value":0},
		{"id":"DECLINE","value":0},
		{"id":"MEXAMER","value":0},
		{"id":"ASIAN","value":0},
		{"id":"EUROPEAN","value":0},
		{"id":"HISPA","value":0},
		{"id":"TWOMORE","value":0},
		{"id":"CENTAMER","value":0},
		{"id":"NORESPON","value":0},
		{"id":"FILIPINO","value":0},
		{"id":"BLACK","value":0},
		{"id":"CHINESE","value":0},
		{"id":"MEXICAN","value":0},
		{"id":"AMIND","value":0},
		{"id":"OTHRNWT","value":0},
		{"id":"UNKNOWN","value":0},
		{"id":"HISPANIC","value":0},
		{"id":"BLACKPRF","value":0},
		{"id":"OTHRLTHS","value":0},
		{"id":"SOAMER","value":0},
		{"id":"OTHRWHT","value":0},
		{"id":"CUBAN","value":0},
		{"id":"LAOTIAN","value":0},
		{"id":"OTHRSEAS","value":0},
		{"id":"PUERTOR","value":0},
		{"id":"OTHRPACI","value":0},
		{"id":"JAPANESE","value":0},
		{"id":"SAMOAN","value":0},
		{"id":"PACIF","value":0},
		{"id":"KOREAN","value":0},
		{"id":"CAMBOD","value":0},
		{"id":"ASINDIAN","value":0},
		{"id":"OTHRASN","value":0},
		{"id":"HAWAIIAN","value":0},
		{"id":"VIETNAME","value":0},
		{"id":"AFRAM","value":0},
		{"id":"GUAMANIA","value":0},
		{"id":"THAI","value":0},
		{"id":"DECHISPA","value":0},
		{"id":"OTHER","value":0},
		{"id":"OTHRAMIN","value":0},
		{"id":"HMONG","value":0}
	];

	//Processing data into a chartable form
	SD.forEach(function(d) {
		if (d.gender == 'M')
			++GenderData[0].value;
		else if (d.gender == 'F') ++GenderData[1].value;

		var index = Ethnicities.indexOf(d.ethnicity);
		if (index != -1) 
			++EthnicityData[index].value;

		index = Grades.indexOf(d.grade);
		if (index != -1) 
			++GradeData[index].value;
		
	});

	//Rid ourselves of empty data points
	GradeData = GradeData.filter(function (d) {
		if (d.value != 0) return d;
		return;
	});
	GenderData = GenderData.filter(function (d) {
		if (d.value != 0) return d;
		return;
	});
	EthnicityData = EthnicityData.filter(function (d) {
		if (d.value != 0) return d;
		return;
	});

	//Use the generated data to make the pies
	MakePie(GradeCont,GradeData,radius,padding);
	MakePie(GenderCont,GenderData,radius,padding);
	MakePie(EthnicityCont,EthnicityData,radius,padding);


	return base;
}
function MakePie(Container,data,radius,padding) {
	//Creates a pie chart in 'container'

	Container.data([data]);
	var color = d3.scale.category20();
	var pie = d3.layout.pie()
		.value(
			function(d) {
				return d.value;
			}
		)
		.sort(
			null
		)
	;
	var arcs = Container.selectAll("g.slice")
		.data(pie)
		.enter()
			.append("svg:g")
				.attr("class", "slice")
	;
	//Create the drawn data for each pie slice
	arcs.append("svg:path")
		.attr("fill", 
			function(d, i) {
				return color(i);
			}
		)
		.attr("d", 
			//Set the radius of each slice, to indicate the 'fullness' of the class it represents
			d3.svg.arc().outerRadius(
				function() {
					return radius;
				}
			)
		)
		.append("svg:title")
			.text(
				function(d,i) { return data[i].value + " " + data[i].id;
			})

	;

	//Add text labels to each pie slice if needed
	arcs.append("svg:text")
		.attr("class","tinytext")
		.attr("transform", 
			function(d) {
				var rotation = 
					(
						(
							(d.startAngle + d.endAngle)
							/
							(4*Math.PI)
						) -.25
					) * 360
				;
				if (rotation > 90 && rotation < 270) {
					return "rotate(" 
						+ 
						(360 - (180 - rotation))
						+ ") translate(" + (-radius - 4) + ",0)"
					;
				}
				else {
					return "rotate(" 
						+ 
						rotation
						+ ") translate(" + (radius + 4) + ",0)"
					;
				}
			}
		)
		.attr("text-anchor", function(d) {
				var rotation = 
					(
						(
							(d.startAngle + d.endAngle)
							/
							(4*Math.PI)
						) -.25
					) * 360
				;
				if (rotation > 90 && rotation < 270) return "end";
				
				else return "start";
				
			}
		)
		.text(
			function(d,i) { 
				return data[i].id;
			}
		)
	;
}


function mouseUp() {
	var div = document.getElementById('tooltip');
	if (!div) return;
	Pie_TooltipInitialPos.x = Number(div.style.left.substr(0,div.style.left.length-2));
	Pie_TooltipInitialPos.y = Number(div.style.top.substr(0,div.style.top.length-2));
    window.removeEventListener('mousemove', tooltipDrag, true);
}

function mouseDown(e) {
	Pie_TooltipStartDrag.x = e.clientX + window.scrollX;
	Pie_TooltipStartDrag.y = e.clientY + window.scrollY;
	window.addEventListener('mousemove', tooltipDrag, true);
}

function tooltipDrag(e){
    var div = document.getElementById('tooltip');
	div.style.position = 'absolute';
	div.style.top = (Pie_TooltipInitialPos.y + (e.clientY - (Pie_TooltipStartDrag.y - window.scrollY)))  + 'px';
	div.style.left = (Pie_TooltipInitialPos.x + (e.clientX - (Pie_TooltipStartDrag.x - window.scrollX)))  + 'px';
}